@@ -54,6 +54,7 @@ class WERBPE(Metric):
         use_cer: Whether to compute word-error-rate or character-error-rate.
         ctc_decode: Whether to perform CTC decode.
         log_prediction: Whether to log a single decoded sample per call.
+        fold_consecutive: Whether repeated consecutive tokens should be folded into one when decoding.
 
     Returns:
         res: a tuple of 3 zero dimensional float32 ``torch.Tensor` objects: a WER score, a sum of Levenstein's
@@ -68,6 +69,7 @@ class WERBPE(Metric):
         ctc_decode=True,
         log_prediction=True,
         dist_sync_on_step=False,
+        fold_consecutive=True,
     ):
         super().__init__(dist_sync_on_step=dist_sync_on_step, compute_on_step=False)
         self.tokenizer = tokenizer
@@ -76,6 +78,7 @@ class WERBPE(Metric):
         self.use_cer = use_cer
         self.ctc_decode = ctc_decode
         self.log_prediction = log_prediction
+        self.fold_consecutive = fold_consecutive
 
         self.add_state("scores", default=torch.tensor(0), dist_reduce_fx='sum', persistent=False)
         self.add_state("words", default=torch.tensor(0), dist_reduce_fx='sum', persistent=False)
@@ -109,16 +112,22 @@ class WERBPE(Metric):
         prediction_cpu_tensor = predictions.long().cpu()
         # iterate over batch
         for ind in range(prediction_cpu_tensor.shape[0]):
-            prediction = prediction_cpu_tensor[ind].detach().numpy().tolist()
-            if predictions_len is not None:
-                prediction = prediction[: predictions_len[ind]]
-            # CTC decoding procedure
-            decoded_prediction = []
-            previous = self.blank_id
-            for p in prediction:
-                if (p != previous or previous == self.blank_id) and p != self.blank_id:
-                    decoded_prediction.append(p)
-                previous = p
+            if self.fold_consecutive:
+                prediction = prediction_cpu_tensor[ind].detach().numpy().tolist()
+                if predictions_len is not None:
+                    prediction = prediction[: predictions_len[ind]]
+                # CTC decoding procedure
+                decoded_prediction = []
+                previous = self.blank_id
+                for p in prediction:
+                    if (p != previous or previous == self.blank_id) and p != self.blank_id:
+                        decoded_prediction.append(p)
+                    previous = p
+            else:
+                prediction = prediction_cpu_tensor[ind].detach()
+                if predictions_len is not None:
+                    prediction = prediction[: predictions_len[ind]]
+                decoded_prediction = prediction[prediction != self.blank_id].tolist()
 
             text = self.decode_tokens_to_str(decoded_prediction)
 
