@@ -12,15 +12,12 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 import logging
-import os
-import re
 from abc import ABC, abstractmethod
 from typing import List, Optional, Union
 
 import torch
-from omegaconf import OmegaConf
 
-from nemo.core.classes import ModelPT, exportable, typecheck
+from nemo.core.classes import ModelPT
 from nemo.core.classes.exportable import Exportable
 from nemo.utils import model_utils
 
@@ -65,6 +62,43 @@ class ASRModel(ModelPT, ABC):
         list_of_models = model_utils.resolve_subclass_pretrained_model_info(cls)
         return list_of_models
 
+    def setup_optimization_flags(self):
+        """
+        Utility method that must be explicitly called by the subclass in order to support optional optimization flags.
+        This method is the only valid place to access self.cfg prior to DDP training occurs.
+
+        The subclass may chose not to support this method, therefore all variables here must be checked via hasattr()
+        """
+        # Skip update if nan/inf grads appear on any rank.
+        self._skip_nan_grad = False
+        if "skip_nan_grad" in self._cfg and self._cfg["skip_nan_grad"]:
+            self._skip_nan_grad = self._cfg["skip_nan_grad"]
+
+    def on_after_backward(self):
+        """
+        zero-out the gradients which any of them is NAN or INF
+        """
+        super().on_after_backward()
+
+        if hasattr(self, '_skip_nan_grad') and self._skip_nan_grad:
+            device = next(self.parameters()).device
+            valid_gradients = torch.tensor([1], device=device, dtype=torch.float32)
+
+            # valid_gradients = True
+            for param_name, param in self.named_parameters():
+                if param.grad is not None:
+                    is_not_nan_or_inf = not (torch.isnan(param.grad).any() or torch.isinf(param.grad).any())
+                    if not is_not_nan_or_inf:
+                        valid_gradients = valid_gradients * 0
+                        break
+
+            if torch.distributed.is_initialized():
+                torch.distributed.all_reduce(valid_gradients, op=torch.distributed.ReduceOp.MIN)
+
+            if valid_gradients < 1:
+                logging.warning(f'detected inf or nan values in gradients! Setting gradients to zero.')
+                self.zero_grad()
+
 
 class ExportableEncDecModel(Exportable):
     """
