@@ -115,7 +115,7 @@ def write_vad_infer_manifest(file, args_func):
     try:
         sr = 16000
         x, _sr = librosa.load(filepath, sr=sr, offset=in_offset, duration=in_duration)
-        duration = librosa.get_duration(x, sr=sr)
+        duration = librosa.get_duration(y=x, sr=sr)
         left = duration
         current_offset = in_offset
 
@@ -161,7 +161,7 @@ def write_vad_infer_manifest(file, args_func):
     except Exception as e:
         err_file = "error.log"
         with open(err_file, 'w', encoding='utf-8') as fout:
-            fout.write(file + ":" + str(e))
+            fout.write(filepath + ":" + str(e))
     return res
 
 
@@ -748,7 +748,7 @@ def plot(
     FRAME_LEN = 0.01
 
     audio, sample_rate = librosa.load(path=path2audio_file, sr=16000, mono=True, offset=offset, duration=duration)
-    dur = librosa.get_duration(audio, sr=sample_rate)
+    dur = librosa.get_duration(y=audio, sr=sample_rate)
 
     time = np.arange(offset, offset + dur, FRAME_LEN)
     frame = np.loadtxt(path2_vad_pred)
@@ -881,3 +881,113 @@ def init_vad_model(model_path):
         logging.info(f"Using NGC cloud VAD model {model_path}")
         vad_model = EncDecClassificationModel.from_pretrained(model_name=model_path)
     return vad_model
+
+
+def stitch_segmented_asr_output(
+    segmented_output_manifest: str,
+    speech_segments_tensor_dir: str = "speech_segments",
+    stitched_output_manifest: str = "asr_stitched_output_manifest.json",
+) -> str:
+    """
+    Stitch the prediction of speech segments.
+    """
+    if not os.path.exists(speech_segments_tensor_dir):
+        os.mkdir(speech_segments_tensor_dir)
+
+    segmented_output = []
+    for line in open(segmented_output_manifest, 'r', encoding='utf-8'):
+        file = json.loads(line)
+        segmented_output.append(file)
+
+    with open(stitched_output_manifest, 'w', encoding='utf-8') as fout:
+        speech_segments = torch.Tensor()
+        all_pred_text = ""
+        if len(segmented_output) > 1:
+            for i in range(1, len(segmented_output)):
+                start, end = (
+                    segmented_output[i - 1]['offset'],
+                    segmented_output[i - 1]['offset'] + segmented_output[i - 1]['duration'],
+                )
+                new_seg = torch.tensor([start, end]).unsqueeze(0)
+                speech_segments = torch.cat((speech_segments, new_seg), 0)
+                pred_text = segmented_output[i - 1]['pred_text']
+                all_pred_text += pred_text
+                name = segmented_output[i - 1]['audio_filepath'].split("/")[-1].rsplit(".", 1)[0]
+
+                if segmented_output[i - 1]['audio_filepath'] != segmented_output[i]['audio_filepath']:
+
+                    speech_segments_tensor_path = os.path.join(speech_segments_tensor_dir, name + '.pt')
+                    torch.save(speech_segments, speech_segments_tensor_path)
+                    meta = {
+                        'audio_filepath': segmented_output[i - 1]['audio_filepath'],
+                        'speech_segments_filepath': speech_segments_tensor_path,
+                        'pred_text': all_pred_text,
+                    }
+
+                    json.dump(meta, fout)
+                    fout.write('\n')
+                    fout.flush()
+                    speech_segments = torch.Tensor()
+                    all_pred_text = ""
+                else:
+                    all_pred_text += " "
+        else:
+            i = -1
+
+        start, end = segmented_output[i]['offset'], segmented_output[i]['offset'] + segmented_output[i]['duration']
+        new_seg = torch.tensor([start, end]).unsqueeze(0)
+        speech_segments = torch.cat((speech_segments, new_seg), 0)
+        pred_text = segmented_output[i]['pred_text']
+        all_pred_text += pred_text
+        name = segmented_output[i]['audio_filepath'].split("/")[-1].rsplit(".", 1)[0]
+        speech_segments_tensor_path = os.path.join(speech_segments_tensor_dir, name + '.pt')
+        torch.save(speech_segments, speech_segments_tensor_path)
+
+        meta = {
+            'audio_filepath': segmented_output[i]['audio_filepath'],
+            'speech_segments_filepath': speech_segments_tensor_path,
+            'pred_text': all_pred_text,
+        }
+        json.dump(meta, fout)
+        fout.write('\n')
+        fout.flush()
+
+        logging.info(
+            f"Finish stitch segmented ASR output to {stitched_output_manifest}, the speech segments info has been stored in directory {speech_segments_tensor_dir}"
+        )
+        return stitched_output_manifest
+
+
+def contruct_manfiest_eval(
+    input_manifest: str, stitched_output_manifest: str, aligned_vad_asr_output_manifest: str = "vad_asr_out.json"
+) -> str:
+
+    """
+    Generate aligned manifest for evaluation.
+    Because some pure noise samples might not appears in stitched_output_manifest.
+    """
+    stitched_output = dict()
+    for line in open(stitched_output_manifest, 'r', encoding='utf-8'):
+        file = json.loads(line)
+        stitched_output[file["audio_filepath"]] = file
+
+    out = []
+    for line in open(input_manifest, 'r', encoding='utf-8'):
+        file = json.loads(line)
+        sample = file["audio_filepath"]
+        if sample in stitched_output:
+            file["pred_text"] = stitched_output[sample]["pred_text"]
+            file["speech_segments_filepath"] = stitched_output[sample]["speech_segments_filepath"]
+        else:
+            file["pred_text"] = ""
+            file["speech_segments_filepath"] = ""
+
+        out.append(file)
+
+    with open(aligned_vad_asr_output_manifest, 'w', encoding='utf-8') as fout:
+        for i in out:
+            json.dump(i, fout)
+            fout.write('\n')
+            fout.flush()
+
+    return aligned_vad_asr_output_manifest
