@@ -33,6 +33,7 @@ def rnn(
     t_max: Optional[int] = None,
     weights_init_scale: float = 1.0,
     hidden_hidden_bias_scale: float = 0.0,
+    proj_size: int = 0,
 ) -> torch.nn.Module:
     """
     Utility function to provide unified interface to common LSTM RNN modules.
@@ -84,6 +85,7 @@ def rnn(
             t_max=t_max,
             weights_init_scale=weights_init_scale,
             hidden_hidden_bias_scale=hidden_hidden_bias_scale,
+            proj_size=proj_size,
         )
 
     if norm == "batch":
@@ -98,11 +100,12 @@ def rnn(
             norm_first_rnn=norm_first_rnn,
             weights_init_scale=weights_init_scale,
             hidden_hidden_bias_scale=hidden_hidden_bias_scale,
+            proj_size=proj_size,
         )
 
     if norm == "layer":
         return torch.jit.script(
-            ln_lstm(  # torch.jit.script(
+            ln_lstm(
                 input_size=input_size,
                 hidden_size=hidden_size,
                 num_layers=num_layers,
@@ -156,6 +159,7 @@ class LSTMDropout(torch.nn.Module):
         t_max: Optional[int] = None,
         weights_init_scale: float = 1.0,
         hidden_hidden_bias_scale: float = 0.0,
+        proj_size: int = 0,
     ):
         """Returns an LSTM with forget gate bias init to `forget_gate_bias`.
         Args:
@@ -187,7 +191,7 @@ class LSTMDropout(torch.nn.Module):
         super(LSTMDropout, self).__init__()
 
         self.lstm = torch.nn.LSTM(
-            input_size=input_size, hidden_size=hidden_size, num_layers=num_layers, dropout=dropout,
+            input_size=input_size, hidden_size=hidden_size, num_layers=num_layers, dropout=dropout, proj_size=proj_size
         )
 
         if t_max is not None:
@@ -244,6 +248,7 @@ class RNNLayer(torch.nn.Module):
         t_max: Optional[int] = None,
         weights_init_scale: float = 1.0,
         hidden_hidden_bias_scale: float = 0.0,
+        proj_size: int = 0,
     ):
         super().__init__()
 
@@ -261,6 +266,7 @@ class RNNLayer(torch.nn.Module):
                 t_max=t_max,
                 weights_init_scale=weights_init_scale,
                 hidden_hidden_bias_scale=hidden_hidden_bias_scale,
+                proj_size=proj_size,
             )
         else:
             self.rnn = rnn_type(input_size=input_size, hidden_size=hidden_size, bias=not batch_norm)
@@ -299,6 +305,7 @@ class BNRNNSum(torch.nn.Module):
         t_max: Optional[int] = None,
         weights_init_scale: float = 1.0,
         hidden_hidden_bias_scale: float = 0.0,
+        proj_size: int = 0,
     ):
         super().__init__()
         self.rnn_layers = rnn_layers
@@ -317,6 +324,7 @@ class BNRNNSum(torch.nn.Module):
                     t_max=t_max,
                     weights_init_scale=weights_init_scale,
                     hidden_hidden_bias_scale=hidden_hidden_bias_scale,
+                    proj_size=proj_size,
                 )
             )
 
