@@ -20,6 +20,7 @@ from pytorch_lightning.trainer.trainer import Trainer
 
 from nemo.collections.nlp.modules.common.megatron.megatron_init import initialize_model_parallel_for_nemo
 from nemo.utils import AppState, logging
+from nemo.utils.get_rank import is_global_rank_zero
 
 
 class MegatronDataset(torch.utils.data.Dataset):
@@ -36,8 +37,8 @@ class MegatronDataset(torch.utils.data.Dataset):
             )
             app_state.global_rank = trainer.global_rank
             app_state.world_size = trainer.world_size
-            app_state.model_parallel_size = 1
-            app_state.model_parallel_rank = trainer.global_rank
+            app_state.tensor_model_parallel_size = 1
+            app_state.tensor_model_parallel_rank = trainer.global_rank
 
             initialize_model_parallel_for_nemo(
                 world_size=trainer.world_size,
@@ -50,9 +51,15 @@ class MegatronDataset(torch.utils.data.Dataset):
         try:
             from nemo.collections.nlp.data.language_modeling.megatron.dataset_utils import compile_helper
 
-            compile_helper()
-            logging.info('Megatron dataset helper compiled successfully.')
+            if is_global_rank_zero():
+                compile_helper()
+
+            if torch.distributed.is_available() and torch.distributed.is_initialized():
+                torch.distributed.barrier()
+
             from nemo.collections.nlp.data.language_modeling.megatron import helpers
+
+            logging.info('Megatron dataset helper compiled successfully.')
         except ImportError:
             raise ImportError(
                 f'Could not compile megatron dataset C++ helper functions and therefore cannot import helpers python file.'
