@@ -21,7 +21,10 @@ from nemo.collections.nlp.models.duplex_text_normalization import (
 from nemo.collections.nlp.models.entity_linking.entity_linking_model import EntityLinkingModel
 from nemo.collections.nlp.models.glue_benchmark.glue_benchmark_model import GLUEModel
 from nemo.collections.nlp.models.information_retrieval import BertDPRModel, BertJointIRModel
-from nemo.collections.nlp.models.intent_slot_classification import IntentSlotClassificationModel
+from nemo.collections.nlp.models.intent_slot_classification import (
+    IntentSlotClassificationModel,
+    MultiLabelIntentSlotClassificationModel,
+)
 from nemo.collections.nlp.models.language_modeling.bert_lm_model import BERTLMModel
 from nemo.collections.nlp.models.language_modeling.transformer_lm_model import TransformerLMModel
 from nemo.collections.nlp.models.machine_translation import MTEncDecModel
