@@ -55,21 +55,27 @@ class BERTLMModel(ModelPT):
 
     def __init__(self, cfg: DictConfig, trainer: Trainer = None):
 
+        vocab_file = None
+        config_dict = None
+        config_file = None
+
         if cfg.tokenizer is not None:
-            self._setup_tokenizer(cfg.tokenizer)
+            if cfg.tokenizer.get('tokenizer_name') and cfg.tokenizer.get('tokenizer_model'):
+                self._setup_tokenizer(cfg.tokenizer)
+            if cfg.get('tokenizer.vocab_file'):
+                vocab_file = self.register_artifact('tokenizer.vocab_file', cfg.tokenizer.vocab_file)
         else:
             self.tokenizer = None
 
         super().__init__(cfg=cfg, trainer=trainer)
 
+        if cfg.get('language_model.config'):
+            config_dict = OmegaConf.to_container(cfg.language_model.config)
+        if cfg.get('language_model.config_file'):
+            config_file = self.register_artifact('language_model.config_file', cfg.language_model.config_file)
+
         self.bert_model = get_lm_model(
-            pretrained_model_name=cfg.language_model.pretrained_model_name,
-            config_file=self.register_artifact('language_model.config_file', cfg.language_model.config_file),
-            config_dict=OmegaConf.to_container(cfg.language_model.config) if cfg.language_model.config else None,
-            checkpoint_file=cfg.language_model.lm_checkpoint,
-            vocab_file=self.register_artifact('tokenizer.vocab_file', cfg.tokenizer.vocab_file)
-            if cfg.tokenizer
-            else None,
+            config_file=config_file, config_dict=config_dict, vocab_file=vocab_file, trainer=trainer, cfg=cfg,
         )
 
         self.hidden_size = self.bert_model.config.hidden_size
@@ -115,7 +121,7 @@ class BERTLMModel(ModelPT):
         self.setup_optimization(cfg.optim)
 
     @typecheck()
-    def forward(self, input_ids, token_type_ids, attention_mask):
+    def forward(self, input_ids, attention_mask, token_type_ids):
         """
         No special modification required for Lightning, define it as you normally would
         in the `nn.Module` in vanilla PyTorch.
@@ -123,6 +129,9 @@ class BERTLMModel(ModelPT):
         hidden_states = self.bert_model(
             input_ids=input_ids, token_type_ids=token_type_ids, attention_mask=attention_mask,
         )
+        if isinstance(hidden_states, tuple):
+            hidden_states = hidden_states[0]
+
         mlm_log_probs = self.mlm_classifier(hidden_states=hidden_states)
         if self.only_mlm_loss:
             return (mlm_log_probs,)
