@@ -12,7 +12,9 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-
+from nemo.collections.nlp.models.token_classification.punctuation_capitalization_config import (
+    PunctuationCapitalizationModelConfig,
+)
 from nemo.collections.nlp.models.token_classification.punctuation_capitalization_model import (
     PunctuationCapitalizationModel,
 )
