@@ -26,12 +26,12 @@ from pathlib import Path
 from typing import Dict, List, Optional, Union
 
 import hydra
-import pytorch_lightning as pl
 import wrapt
+from huggingface_hub import hf_hub_download
+from huggingface_hub.hf_api import HfFolder
 from omegaconf import DictConfig, OmegaConf
 
 import nemo
-from nemo.core.config import TrainerConfig
 from nemo.core.connectors.save_restore_connector import SaveRestoreConnector
 from nemo.core.neural_types import NeuralType, NeuralTypeComparisonResult
 from nemo.utils import logging, model_utils
@@ -458,17 +458,6 @@ class Serialization(ABC):
                         imported_cls = cls
                     accepts_trainer = Serialization._inspect_signature_for_trainer(imported_cls)
                     if accepts_trainer:
-                        if trainer is None:
-                            # Create a dummy PL trainer object
-                            cfg_trainer = TrainerConfig(
-                                gpus=1,
-                                accelerator="ddp",
-                                num_nodes=1,
-                                # Need to set the following two to False as ExpManager will take care of them differently.
-                                logger=False,
-                                checkpoint_callback=False,
-                            )
-                            trainer = pl.Trainer(cfg_trainer)
                         instance = imported_cls(cfg=config, trainer=trainer)
                     else:
                         instance = imported_cls(cfg=config)
@@ -682,8 +671,48 @@ class Model(Typing, Serialization, FileIO):
         if save_restore_connector is None:
             save_restore_connector = SaveRestoreConnector()
 
+        # Resolve if the pretrained model name is from NGC or other sources
+        # HF Hub source
+        if '/' in model_name:
+            class_, nemo_model_file_in_cache = cls._get_hf_hub_pretrained_model_info(
+                model_name=model_name, refresh_cache=refresh_cache
+            )
+        else:
+            # NGC source
+            class_, nemo_model_file_in_cache = cls._get_ngc_pretrained_model_info(
+                model_name=model_name, refresh_cache=refresh_cache
+            )
+
+        instance = class_.restore_from(
+            restore_path=nemo_model_file_in_cache,
+            override_config_path=override_config_path,
+            map_location=map_location,
+            strict=strict,
+            return_config=return_config,
+            trainer=trainer,
+            save_restore_connector=save_restore_connector,
+        )
+        return instance
+
+    @classmethod
+    def _get_ngc_pretrained_model_info(cls, model_name: str, refresh_cache: bool = False) -> (type, str):
+        """
+        Resolve the NGC model pretrained information given a model name.
+        Assumes the model subclass implements the `list_available_models()` inherited method.
+
+        Args:
+            model_name: Str name of the model. Must be the original name or an alias of the model, without any '/'.
+            refresh_cache: Bool, determines whether cache must be refreshed (model is re-downloaded).
+
+        Returns:
+            A tuple of details describing :
+            -   The resolved class of the model. This requires subclass to implement PretrainedModelInfo.class_.
+                If the class cannot be resolved, default to the class that called this method.
+            -   The path to the NeMo model (.nemo file) in some cached directory.
+        """
         location_in_the_cloud = None
         description = None
+        class_ = None
         models = cls.list_available_models()
         if models is not None:
             for pretrained_model_info in cls.list_available_models():
@@ -714,19 +743,53 @@ class Model(Typing, Serialization, FileIO):
         nemo_model_file_in_cache = maybe_download_from_cloud(
             url=url, filename=filename, cache_dir=cache_dir, subfolder=cache_subfolder, refresh_cache=refresh_cache
         )
+
         logging.info("Instantiating model from pre-trained checkpoint")
+
         if class_ is None:
             class_ = cls
-        instance = class_.restore_from(
-            restore_path=nemo_model_file_in_cache,
-            override_config_path=override_config_path,
-            map_location=map_location,
-            strict=strict,
-            return_config=return_config,
-            trainer=trainer,
-            save_restore_connector=save_restore_connector,
+
+        return class_, nemo_model_file_in_cache
+
+    @classmethod
+    def _get_hf_hub_pretrained_model_info(cls, model_name: str, refresh_cache: bool = False) -> (type, str):
+        """
+        Resolve the HuggingFace Hub model pretrained information given a model name.
+        The model name must be of general syntax ``{source_repo}/{model_name}``.
+
+        Note:
+            The ``{source_repo}`` need not be ``nvidia``, it can be any public repository, even external to Nvidia.
+            This allows public, externally contributed models to be run freely using Nvidia NeMo.
+
+        Args:
+            model_name: Str name of the model. Must be the original name or an alias of the model, without any '/'.
+            refresh_cache: Bool, determines whether cache must be refreshed (model is re-downloaded).
+
+        Returns:
+            A tuple of details describing :
+            -   The resolved class of the model. Since the source is external to NeMo, always default to using
+                the calling class. Depend on target class resolution by restore_from() for calling the correct class.
+            -   The path to the NeMo model (.nemo file) in some cached directory (managed by HF Hub).
+        """
+        # Resolve the model name without origin for filename
+        resolved_model_filename = model_name.split("/")[-1] + '.nemo'
+
+        # Check if api token exists, use if it does
+        is_token_available = HfFolder.get_token() is not None
+
+        # Try to load the model from the Huggingface Hub
+        path = hf_hub_download(
+            repo_id=model_name,
+            filename=resolved_model_filename,
+            force_download=refresh_cache,
+            use_auth_token=is_token_available,
         )
-        return instance
+
+        # Cannot pre-resolve the specific class without double instantiation (first for config, second for model params)
+        # Default to current class, and perform basic class path resolution (handled via restore_from() + target class)
+        class_ = cls
+
+        return class_, path
 
 
 class typecheck:
