@@ -48,6 +48,7 @@ class AppState(metaclass=Singleton):
         self._tensor_model_parallel_group = None
         self._pipeline_model_parallel_size = None
         self._pipeline_model_parallel_group = None
+        self._pipeline_model_parallel_split_rank = None
         self._is_megatron_initialized = False
         self._data_parallel_size = None
         self._data_parallel_group = None
@@ -248,6 +249,22 @@ class AppState(metaclass=Singleton):
         """
         self._pipeline_model_parallel_rank = rank
 
+    @property
+    def pipeline_model_parallel_split_rank(self):
+        """ Property returns the rank at which Encoder and Decoder are split into different pipelines for Megatrron Encoder-Decoder models.
+            Returns:
+                Pipeline model parallel split rank.
+        """
+        return self._pipeline_model_parallel_split_rank
+
+    @pipeline_model_parallel_split_rank.setter
+    def pipeline_model_parallel_split_rank(self, rank):
+        """ Property sets the rank at which Encoder and Decoder are split into different pipelines for Megatrron Encoder-Decoder models.
+            Args:
+                rank (int): Model parallel split rank.
+        """
+        self._pipeline_model_parallel_split_rank = rank
+
     @property
     def pipeline_model_parallel_group(self):
         """ Property returns the model parallel group.
