@@ -175,6 +175,15 @@ def parse_dataset_as_name(name: str) -> str:
     if 'dataset' in name:
         name = name.replace('dataset', '')
 
+    # Test if the manifes/dataset name was simply `manifest.yaml` or `dataset.yaml`: Invalid names.
+    if name == '':
+        raise ValueError(
+            "Provided dataset / manifest filename was `manifest.json` or `dataset.json`.\n"
+            "Such a name is invalid, since multiple datasets/manifests can share the same name,\n"
+            "thereby overriding their results during logging. Please pick a more discriptive filename \n"
+            "for the provided dataset / manifest file."
+        )
+
     if '_' != name[-1]:
         name = name + '_'
 
@@ -575,11 +584,25 @@ def resolve_cache_dir() -> Path:
     return path
 
 
+def uninject_model_parallel_rank(filepath):
+    filepath = str(filepath)
+    if 'mp_rank' in filepath or 'tp_rank' in filepath:
+        dirname = os.path.dirname(os.path.dirname(filepath))
+        basename = os.path.basename(filepath)
+        filepath = os.path.join(dirname, basename)
+        return filepath
+    else:
+        return filepath
+
+
 def inject_model_parallel_rank(filepath):
     """
     Injects tensor/pipeline model parallel ranks into the filepath.
     Does nothing if not using model parallelism.
     """
+    # first make sure filepath does not have rank
+    filepath = uninject_model_parallel_rank(filepath)
+
     app_state = AppState()
     if app_state.model_parallel_size is not None and app_state.model_parallel_size > 1:
         # filepath needs to be updated to include mp_rank
@@ -592,13 +615,3 @@ def inject_model_parallel_rank(filepath):
         return filepath
     else:
         return filepath
-
-
-def uninject_model_parallel_rank(filepath):
-    if 'mp_rank' or 'tp_rank' in filepath:
-        dirname = os.path.dirname(os.path.dirname(filepath))
-        basename = os.path.basename(filepath)
-        filepath = os.path.join(dirname, basename)
-        return filepath
-    else:
-        return filepath
