@@ -16,6 +16,7 @@ from argparse import ArgumentParser
 from time import perf_counter
 from typing import List
 
+from nemo_text_processing.text_normalization.data_loader_utils import check_installation, get_installation_msg
 from nemo_text_processing.text_normalization.normalize import Normalizer
 from nemo_text_processing.text_normalization.token_parser import TokenParser
 
@@ -32,6 +33,9 @@ class InverseNormalizer(Normalizer):
     """
 
     def __init__(self, lang: str = 'en', cache_dir: str = None, overwrite_cache: bool = False):
+
+        if not check_installation():
+            raise ImportError(get_installation_msg())
         if lang == 'en':
             from nemo_text_processing.inverse_text_normalization.en.taggers.tokenize_and_classify import ClassifyFst
             from nemo_text_processing.inverse_text_normalization.en.verbalizers.verbalize_final import (
